'use strict';

const packageName = require('../package.json').name;
const path = require('path');
const debug = require('debug')(packageName);
const errors = require('blobbr-pipeline-errors');
const BadConfig = errors.BadConfig;
const UnexpectedPipelineAbort = errors.UnexpectedPipelineAbort;
const fs = require('fs');
const Chance = require('chance');
const randomGenerator = new Chance();

module.exports = config => {
  debug('generating a new processor from config');
  debug('Checking if the directory exists');
  let stat = null;
  try {
    stat = fs.statSync(config.directory);
  } catch (e) {
    throw new BadConfig(packageName, 1,
      `${config.directory} does not exists`, {});
  }
  if (!stat.isDirectory()) {
    throw new BadConfig(packageName, 2,
      `${config.directory} is not a valid directory`, {});
  }

  const processor = function(dependencies) {
    const logger = (message, level = 'debug') => {
      dependencies.log(message, level);
      debug(message);
    };
    const individualWrites = dependencies.files.map(file => {
      const fileName = `${Date.now()}-${randomGenerator.guid()}-${file.name}`;
      const fullPath = path.resolve(config.directory, fileName);
      return new Promise((resolve, reject) => {
        fs.writeFile(fullPath, file.buffer, error => {
          if (error) {
            reject(new UnexpectedPipelineAbort(packageName, 1,
              `Impossible to write ${fullPath}`, {
                underlyingError: error
              }));
          } else {
            resolve(fileName);
          }
        });
      });
    });
    return Promise.all(individualWrites)
      .then(files => {
        logger(`All files were written on disk`);
        return {
          output: {
            'blobbr-write-to-disk': {
              files
            }
          }
        };
      });
  };
  // Here the input and output schemas are constant
  // but you can make them depend on the config
  processor.inputSchema = require('blobbr-body-reader/output-schema.json');
  processor.inputSchema.properties.log = {
    required: true,
    description: 'This is the logger function',
    type: 'function'
  };
  processor.outputSchema = require('../output-schema.json');
  return processor;
};

// The config must always be the same
module.exports.configSchema = require('../config-schema.json');
